@@ -1,9 +1,9 @@
 /* example.c -- usage example of the zlib compression library
- * Copyright (C) 1995-1996 Jean-loup Gailly.
+ * Copyright (C) 1995-1998 Jean-loup Gailly.
  * For conditions of distribution and use, see copyright notice in zlib.h 
  */
 
-/* $Id: example.c,v 1.1.1.1 1996/04/18 19:36:46 mdorman Exp $ */
+/* @(#) $Id$ */
 
 #include <stdio.h>
 #include "zlib.h"
@@ -30,30 +30,30 @@ const char hello[] = "hello, hello!";
 const char dictionary[] = "hello";
 uLong dictId; /* Adler32 value of the dictionary */
 
-void test_compress      OF((Bytef *compr, uLong comprLen,
-		            Bytef *uncompr, uLong uncomprLen));
+void test_compress      OF((Byte *compr, uLong comprLen,
+		            Byte *uncompr, uLong uncomprLen));
 void test_gzio          OF((const char *out, const char *in, 
-		            Bytef *uncompr, int uncomprLen));
-void test_deflate       OF((Bytef *compr, uLong comprLen));
-void test_inflate       OF((Bytef *compr, uLong comprLen,
-		            Bytef *uncompr, uLong uncomprLen));
-void test_large_deflate OF((Bytef *compr, uLong comprLen,
-		            Bytef *uncompr, uLong uncomprLen));
-void test_large_inflate OF((Bytef *compr, uLong comprLen,
-		            Bytef *uncompr, uLong uncomprLen));
-void test_flush         OF((Bytef *compr, uLong comprLen));
-void test_sync          OF((Bytef *compr, uLong comprLen,
-		            Bytef *uncompr, uLong uncomprLen));
-void test_dict_deflate  OF((Bytef *compr, uLong comprLen));
-void test_dict_inflate  OF((Bytef *compr, uLong comprLen,
-		            Bytef *uncompr, uLong uncomprLen));
+		            Byte *uncompr, int uncomprLen));
+void test_deflate       OF((Byte *compr, uLong comprLen));
+void test_inflate       OF((Byte *compr, uLong comprLen,
+		            Byte *uncompr, uLong uncomprLen));
+void test_large_deflate OF((Byte *compr, uLong comprLen,
+		            Byte *uncompr, uLong uncomprLen));
+void test_large_inflate OF((Byte *compr, uLong comprLen,
+		            Byte *uncompr, uLong uncomprLen));
+void test_flush         OF((Byte *compr, uLong *comprLen));
+void test_sync          OF((Byte *compr, uLong comprLen,
+		            Byte *uncompr, uLong uncomprLen));
+void test_dict_deflate  OF((Byte *compr, uLong comprLen));
+void test_dict_inflate  OF((Byte *compr, uLong comprLen,
+		            Byte *uncompr, uLong uncomprLen));
 int  main               OF((int argc, char *argv[]));
 
 /* ===========================================================================
  * Test compress() and uncompress()
  */
 void test_compress(compr, comprLen, uncompr, uncomprLen)
-    Bytef *compr, *uncompr;
+    Byte *compr, *uncompr;
     uLong comprLen, uncomprLen;
 {
     int err;
@@ -69,6 +69,7 @@ void test_compress(compr, comprLen, uncompr, uncomprLen)
 
     if (strcmp((char*)uncompr, hello)) {
         fprintf(stderr, "bad uncompress\n");
+	exit(1);
     } else {
         printf("uncompress(): %s\n", uncompr);
     }
@@ -80,22 +81,29 @@ void test_compress(compr, comprLen, uncompr, uncomprLen)
 void test_gzio(out, in, uncompr, uncomprLen)
     const char *out; /* output file */
     const char *in;  /* input file */
-    Bytef *uncompr;
+    Byte *uncompr;
     int  uncomprLen;
 {
     int err;
     int len = strlen(hello)+1;
     gzFile file;
+    z_off_t pos;
 
     file = gzopen(out, "wb");
     if (file == NULL) {
         fprintf(stderr, "gzopen error\n");
         exit(1);
     }
-
-    if (gzwrite(file, (const voidp)hello, (unsigned)len) != len) {
-        fprintf(stderr, "gzwrite err: %s\n", gzerror(file, &err));
+    gzputc(file, 'h');
+    if (gzputs(file, "ello") != 4) {
+        fprintf(stderr, "gzputs err: %s\n", gzerror(file, &err));
+	exit(1);
     }
+    if (gzprintf(file, ", %s!", "hello") != 8) {
+        fprintf(stderr, "gzprintf err: %s\n", gzerror(file, &err));
+	exit(1);
+    }
+    gzseek(file, 1L, SEEK_CUR); /* add one zero byte */
     gzclose(file);
 
     file = gzopen(in, "rb");
@@ -107,21 +115,48 @@ void test_gzio(out, in, uncompr, uncomprLen)
     uncomprLen = gzread(file, uncompr, (unsigned)uncomprLen);
     if (uncomprLen != len) {
         fprintf(stderr, "gzread err: %s\n", gzerror(file, &err));
+	exit(1);
     }
-    gzclose(file);
-
     if (strcmp((char*)uncompr, hello)) {
-        fprintf(stderr, "bad gzread\n");
+        fprintf(stderr, "bad gzread: %s\n", (char*)uncompr);
+	exit(1);
     } else {
         printf("gzread(): %s\n", uncompr);
     }
+
+    pos = gzseek(file, -8L, SEEK_CUR);
+    if (pos != 6 || gztell(file) != pos) {
+	fprintf(stderr, "gzseek error, pos=%ld, gztell=%ld\n",
+		pos, gztell(file));
+	exit(1);
+    }
+
+    if (gzgetc(file) != ' ') {
+	fprintf(stderr, "gzgetc error\n");
+	exit(1);
+    }
+
+    gzgets(file, (char*)uncompr, uncomprLen);
+    uncomprLen = strlen((char*)uncompr);
+    if (uncomprLen != 6) { /* "hello!" */
+        fprintf(stderr, "gzgets err after gzseek: %s\n", gzerror(file, &err));
+	exit(1);
+    }
+    if (strcmp((char*)uncompr, hello+7)) {
+        fprintf(stderr, "bad gzgets after gzseek\n");
+	exit(1);
+    } else {
+        printf("gzgets() after gzseek: %s\n", uncompr);
+    }
+
+    gzclose(file);
 }
 
 /* ===========================================================================
  * Test deflate() with small buffers
  */
 void test_deflate(compr, comprLen)
-    Bytef *compr;
+    Byte *compr;
     uLong comprLen;
 {
     z_stream c_stream; /* compression stream */
@@ -159,7 +194,7 @@ void test_deflate(compr, comprLen)
  * Test inflate() with small buffers
  */
 void test_inflate(compr, comprLen, uncompr, uncomprLen)
-    Bytef *compr, *uncompr;
+    Byte *compr, *uncompr;
     uLong comprLen, uncomprLen;
 {
     int err;
@@ -171,12 +206,13 @@ void test_inflate(compr, comprLen, uncompr, uncomprLen)
     d_stream.zfree = (free_func)0;
     d_stream.opaque = (voidpf)0;
 
+    d_stream.next_in  = compr;
+    d_stream.avail_in = 0;
+    d_stream.next_out = uncompr;
+
     err = inflateInit(&d_stream);
     CHECK_ERR(err, "inflateInit");
 
-    d_stream.next_in  = compr;
-    d_stream.next_out = uncompr;
-
     while (d_stream.total_out < uncomprLen && d_stream.total_in < comprLen) {
         d_stream.avail_in = d_stream.avail_out = 1; /* force small buffers */
         err = inflate(&d_stream, Z_NO_FLUSH);
@@ -189,6 +225,7 @@ void test_inflate(compr, comprLen, uncompr, uncomprLen)
 
     if (strcmp((char*)uncompr, hello)) {
         fprintf(stderr, "bad inflate\n");
+	exit(1);
     } else {
         printf("inflate(): %s\n", uncompr);
     }
@@ -198,7 +235,7 @@ void test_inflate(compr, comprLen, uncompr, uncomprLen)
  * Test deflate() with large buffers and dynamic change of compression level
  */
 void test_large_deflate(compr, comprLen, uncompr, uncomprLen)
-    Bytef *compr, *uncompr;
+    Byte *compr, *uncompr;
     uLong comprLen, uncomprLen;
 {
     z_stream c_stream; /* compression stream */
@@ -223,6 +260,7 @@ void test_large_deflate(compr, comprLen, uncompr, uncomprLen)
     CHECK_ERR(err, "deflate");
     if (c_stream.avail_in != 0) {
         fprintf(stderr, "deflate not greedy\n");
+	exit(1);
     }
 
     /* Feed in already compressed data and switch to no compression: */
@@ -242,6 +280,7 @@ void test_large_deflate(compr, comprLen, uncompr, uncomprLen)
     err = deflate(&c_stream, Z_FINISH);
     if (err != Z_STREAM_END) {
         fprintf(stderr, "deflate should report Z_STREAM_END\n");
+	exit(1);
     }
     err = deflateEnd(&c_stream);
     CHECK_ERR(err, "deflateEnd");
@@ -251,7 +290,7 @@ void test_large_deflate(compr, comprLen, uncompr, uncomprLen)
  * Test inflate() with large buffers
  */
 void test_large_inflate(compr, comprLen, uncompr, uncomprLen)
-    Bytef *compr, *uncompr;
+    Byte *compr, *uncompr;
     uLong comprLen, uncomprLen;
 {
     int err;
@@ -263,12 +302,12 @@ void test_large_inflate(compr, comprLen, uncompr, uncomprLen)
     d_stream.zfree = (free_func)0;
     d_stream.opaque = (voidpf)0;
 
-    err = inflateInit(&d_stream);
-    CHECK_ERR(err, "inflateInit");
-
     d_stream.next_in  = compr;
     d_stream.avail_in = (uInt)comprLen;
 
+    err = inflateInit(&d_stream);
+    CHECK_ERR(err, "inflateInit");
+
     for (;;) {
         d_stream.next_out = uncompr;            /* discard the output */
 	d_stream.avail_out = (uInt)uncomprLen;
@@ -282,6 +321,7 @@ void test_large_inflate(compr, comprLen, uncompr, uncomprLen)
 
     if (d_stream.total_out != 2*uncomprLen + comprLen/2) {
         fprintf(stderr, "bad large inflate: %ld\n", d_stream.total_out);
+	exit(1);
     } else {
         printf("large_inflate(): OK\n");
     }
@@ -291,8 +331,8 @@ void test_large_inflate(compr, comprLen, uncompr, uncomprLen)
  * Test deflate() with full flush
  */
 void test_flush(compr, comprLen)
-    Bytef *compr;
-    uLong comprLen;
+    Byte *compr;
+    uLong *comprLen;
 {
     z_stream c_stream; /* compression stream */
     int err;
@@ -308,7 +348,7 @@ void test_flush(compr, comprLen)
     c_stream.next_in  = (Bytef*)hello;
     c_stream.next_out = compr;
     c_stream.avail_in = 3;
-    c_stream.avail_out = (uInt)comprLen;
+    c_stream.avail_out = (uInt)*comprLen;
     err = deflate(&c_stream, Z_FULL_FLUSH);
     CHECK_ERR(err, "deflate");
 
@@ -321,13 +361,15 @@ void test_flush(compr, comprLen)
     }
     err = deflateEnd(&c_stream);
     CHECK_ERR(err, "deflateEnd");
+
+    *comprLen = c_stream.total_out;
 }
 
 /* ===========================================================================
  * Test inflateSync()
  */
 void test_sync(compr, comprLen, uncompr, uncomprLen)
-    Bytef *compr, *uncompr;
+    Byte *compr, *uncompr;
     uLong comprLen, uncomprLen;
 {
     int err;
@@ -339,12 +381,13 @@ void test_sync(compr, comprLen, uncompr, uncomprLen)
     d_stream.zfree = (free_func)0;
     d_stream.opaque = (voidpf)0;
 
+    d_stream.next_in  = compr;
+    d_stream.avail_in = 2; /* just read the zlib header */
+
     err = inflateInit(&d_stream);
     CHECK_ERR(err, "inflateInit");
 
-    d_stream.next_in  = compr;
     d_stream.next_out = uncompr;
-    d_stream.avail_in = 2; /* just read the zlib header */
     d_stream.avail_out = (uInt)uncomprLen;
 
     inflate(&d_stream, Z_NO_FLUSH);
@@ -358,6 +401,7 @@ void test_sync(compr, comprLen, uncompr, uncomprLen)
     if (err != Z_DATA_ERROR) {
         fprintf(stderr, "inflate should report DATA_ERROR\n");
         /* Because of incorrect adler32 */
+	exit(1);
     }
     err = inflateEnd(&d_stream);
     CHECK_ERR(err, "inflateEnd");
@@ -369,7 +413,7 @@ void test_sync(compr, comprLen, uncompr, uncomprLen)
  * Test deflate() with preset dictionary
  */
 void test_dict_deflate(compr, comprLen)
-    Bytef *compr;
+    Byte *compr;
     uLong comprLen;
 {
     z_stream c_stream; /* compression stream */
@@ -396,6 +440,7 @@ void test_dict_deflate(compr, comprLen)
     err = deflate(&c_stream, Z_FINISH);
     if (err != Z_STREAM_END) {
         fprintf(stderr, "deflate should report Z_STREAM_END\n");
+	exit(1);
     }
     err = deflateEnd(&c_stream);
     CHECK_ERR(err, "deflateEnd");
@@ -405,7 +450,7 @@ void test_dict_deflate(compr, comprLen)
  * Test inflate() with a preset dictionary
  */
 void test_dict_inflate(compr, comprLen, uncompr, uncomprLen)
-    Bytef *compr, *uncompr;
+    Byte *compr, *uncompr;
     uLong comprLen, uncomprLen;
 {
     int err;
@@ -417,12 +462,12 @@ void test_dict_inflate(compr, comprLen, uncompr, uncomprLen)
     d_stream.zfree = (free_func)0;
     d_stream.opaque = (voidpf)0;
 
-    err = inflateInit(&d_stream);
-    CHECK_ERR(err, "inflateInit");
-
     d_stream.next_in  = compr;
     d_stream.avail_in = (uInt)comprLen;
 
+    err = inflateInit(&d_stream);
+    CHECK_ERR(err, "inflateInit");
+
     d_stream.next_out = uncompr;
     d_stream.avail_out = (uInt)uncomprLen;
 
@@ -445,6 +490,7 @@ void test_dict_inflate(compr, comprLen, uncompr, uncomprLen)
 
     if (strcmp((char*)uncompr, hello)) {
         fprintf(stderr, "bad inflate with dict\n");
+	exit(1);
     } else {
         printf("inflate with dictionary: %s\n", uncompr);
     }
@@ -458,25 +504,28 @@ int main(argc, argv)
     int argc;
     char *argv[];
 {
-    Bytef *compr, *uncompr;
-    uLong comprLen = 32750*sizeof(int); /* don't overflow on MSDOS */
+    Byte *compr, *uncompr;
+    uLong comprLen = 10000*sizeof(int); /* don't overflow on MSDOS */
     uLong uncomprLen = comprLen;
+    static const char* myVersion = ZLIB_VERSION;
 
-    if (zlib_version[0] != ZLIB_VERSION[0]) {
+    if (zlibVersion()[0] != myVersion[0]) {
         fprintf(stderr, "incompatible zlib version\n");
         exit(1);
 
-    } else if (strcmp(zlib_version, ZLIB_VERSION) != 0) {
+    } else if (strcmp(zlibVersion(), ZLIB_VERSION) != 0) {
         fprintf(stderr, "warning: different zlib version\n");
     }
 
-    compr    = (Bytef*)malloc((uInt)comprLen);
-    uncompr  = (Bytef*)calloc((uInt)uncomprLen, 1); /* must be cleared */
+    compr    = (Byte*)calloc((uInt)comprLen, 1);
+    uncompr  = (Byte*)calloc((uInt)uncomprLen, 1);
+    /* compr and uncompr are cleared to avoid reading uninitialized
+     * data and to ensure that uncompr compresses well.
+     */
     if (compr == Z_NULL || uncompr == Z_NULL) {
         printf("out of memory\n");
 	exit(1);
     }
-
     test_compress(compr, comprLen, uncompr, uncomprLen);
 
     test_gzio((argc > 1 ? argv[1] : "foo.gz"),
@@ -489,8 +538,9 @@ int main(argc, argv)
     test_large_deflate(compr, comprLen, uncompr, uncomprLen);
     test_large_inflate(compr, comprLen, uncompr, uncomprLen);
 
-    test_flush(compr, comprLen);
+    test_flush(compr, &comprLen);
     test_sync(compr, comprLen, uncompr, uncomprLen);
+    comprLen = uncomprLen;
 
     test_dict_deflate(compr, comprLen);
     test_dict_inflate(compr, comprLen, uncompr, uncomprLen);
